package cn.com.infosec.jce.provider;

import cn.com.infosec.crypto.params.DESParameters;
import cn.com.infosec.util.Strings;
import java.math.BigInteger;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.SecureRandom;
import java.security.spec.AlgorithmParameterSpec;
import java.util.Hashtable;
import javax.crypto.KeyAgreementSpi;
import javax.crypto.SecretKey;
import javax.crypto.ShortBufferException;
import javax.crypto.interfaces.DHPrivateKey;
import javax.crypto.interfaces.DHPublicKey;
import javax.crypto.spec.DHParameterSpec;
import javax.crypto.spec.SecretKeySpec;

public class JCEDHKeyAgreement extends KeyAgreementSpi {
   private static final Hashtable algorithms;
   private BigInteger x;
   private BigInteger p;
   private BigInteger g;
   private BigInteger result;
   private SecureRandom random;

   static {
      Hashtable var10000 = algorithms = new Hashtable();
      Integer var0;
      var0 = new Integer.<init>(64);
      Integer var1;
      var1 = new Integer.<init>(192);
      Integer var2;
      var2 = new Integer.<init>(448);
      var10000.put("DES", var0);
      var10000.put("DESEDE", var1);
      var10000.put("BLOWFISH", var2);
   }

   private byte[] bigIntToBytes(BigInteger r) {
      byte[] var3;
      if ((var3 = r.toByteArray())[0] == 0) {
         byte[] var2;
         int r1;
         byte[] var10000 = var2 = new byte[r1 = var3.length - 1];
         System.arraycopy(var3, 1, var2, 0, r1);
         return var10000;
      } else {
         return var3;
      }
   }

   protected Key engineDoPhase(Key key, boolean lastPhase) {
      if (this.x != null) {
         if (key instanceof DHPublicKey) {
            DHPublicKey key1;
            if ((key1 = (DHPublicKey)key).getParams().getG().equals(this.g) && key1.getParams().getP().equals(this.p)) {
               JCEDHKeyAgreement var10000;
               BigInteger var10001;
               JCEDHKeyAgreement var10002;
               BigInteger var3;
               if (lastPhase) {
                  var10000 = this;
                  var10001 = key1.getY();
                  var10002 = this;
                  var3 = this.x;
                  var10000.result = var10001.modPow(var3, var10002.p);
                  return null;
               } else {
                  var10000 = this;
                  var10001 = key1.getY();
                  var10002 = this;
                  var3 = this.x;
                  var10000.result = var3 = var10001.modPow(var3, var10002.p);
                  return new JCEDHPublicKey(var3, key1.getParams());
               }
            } else {
               throw new InvalidKeyException("DHPublicKey not for this KeyAgreement!");
            }
         } else {
            throw new InvalidKeyException("DHKeyAgreement doPhase requires DHPublicKey");
         }
      } else {
         throw new IllegalStateException("Diffie-Hellman not initialised.");
      }
   }

   protected byte[] engineGenerateSecret() {
      if (this.x != null) {
         return this.bigIntToBytes(this.result);
      } else {
         throw new IllegalStateException("Diffie-Hellman not initialised.");
      }
   }

   protected int engineGenerateSecret(byte[] sharedSecret, int offset) {
      if (this.x != null) {
         byte[] var3 = this.bigIntToBytes(this.result);
         if (sharedSecret.length - offset >= var3.length) {
            byte[] var10000 = var3;
            byte[] var10001 = var3;
            int var4 = var3.length;
            System.arraycopy(var10001, 0, sharedSecret, offset, var4);
            return var10000.length;
         } else {
            throw new ShortBufferException("DHKeyAgreement - buffer too short");
         }
      } else {
         throw new IllegalStateException("Diffie-Hellman not initialised.");
      }
   }

   protected SecretKey engineGenerateSecret(String algorithm) {
      if (this.x != null) {
         JCEDHKeyAgreement var10000 = this;
         JCEDHKeyAgreement var10001 = this;
         String var4 = Strings.toUpperCase(algorithm);
         byte[] var2 = var10000.bigIntToBytes(var10001.result);
         Hashtable var3;
         if ((var3 = algorithms).containsKey(var4)) {
            String var6 = var4;
            byte[] var7 = var2;
            int var5;
            var2 = new byte[var5 = (Integer)var3.get(var4) / 8];
            System.arraycopy(var7, 0, var2, 0, var5);
            if (var6.startsWith("DES")) {
               DESParameters.setOddParity(var2);
            }

            return new SecretKeySpec(var2, algorithm);
         } else {
            return new SecretKeySpec(var2, algorithm);
         }
      } else {
         throw new IllegalStateException("Diffie-Hellman not initialised.");
      }
   }

   protected void engineInit(Key key, AlgorithmParameterSpec params, SecureRandom random) {
      if (key instanceof DHPrivateKey) {
         DHPrivateKey key1 = (DHPrivateKey)key;
         this.random = random;
         if (params != null) {
            if (!(params instanceof DHParameterSpec)) {
               throw new InvalidAlgorithmParameterException("DHKeyAgreement only accepts DHParameterSpec");
            }

            DHParameterSpec params1;
            this.p = (params1 = (DHParameterSpec)params).getP();
            this.g = params1.getG();
         } else {
            this.p = key1.getParams().getP();
            this.g = key1.getParams().getG();
         }

         JCEDHKeyAgreement var10000 = this;
         BigInteger var4;
         this.result = var4 = key1.getX();
         var10000.x = var4;
      } else {
         throw new InvalidKeyException("DHKeyAgreement requires DHPrivateKey for initialisation");
      }
   }

   protected void engineInit(Key key, SecureRandom random) {
      if (key instanceof DHPrivateKey) {
         JCEDHKeyAgreement var10000 = this;
         DHPrivateKey key1;
         DHPrivateKey var10002 = key1 = (DHPrivateKey)key;
         this.random = random;
         this.p = key1.getParams().getP();
         this.g = key1.getParams().getG();
         BigInteger var3;
         this.result = var3 = var10002.getX();
         var10000.x = var3;
      } else {
         throw new InvalidKeyException("DHKeyAgreement requires DHPrivateKey");
      }
   }
}
